@@ -148,9 +148,11 @@ else {
 	$dashboard_options = [
 		'max-rows' => DASHBOARD_MAX_ROWS,
 		'max-columns' => DASHBOARD_MAX_COLUMNS,
+		'widget-min-rows' => DASHBOARD_WIDGET_MIN_ROWS,
 		'widget-max-rows' => DASHBOARD_WIDGET_MAX_ROWS,
 		'editable' => $data['dashboard']['editable'],
-		'edit_mode' => $data['dashboard_edit_mode']
+		'edit_mode' => $data['dashboard_edit_mode'],
+		'kioskmode' => ($web_layout_mode === ZBX_LAYOUT_KIOSKMODE)
 	];
 	if ($data['dashboard']['dashboardid'] != 0) {
 		$dashboard_data['id'] = $data['dashboard']['dashboardid'];
@@ -162,7 +164,6 @@ else {
 	// must be done before adding widgets, because it causes dashboard to resize.
 	if ($data['show_timeselector']) {
 		$this->addPostJS(
-			'timeControl.useTimeRefresh('.CWebUser::getRefresh().');'.
 			'timeControl.addObject("scrollbar", '.CJs::encodeJson($data['timeline']).', '.
 				CJs::encodeJson($data['timeControlData']).
 			');'.
@@ -170,10 +171,6 @@ else {
 		);
 	}
 
-	if ($web_layout_mode === ZBX_LAYOUT_KIOSKMODE) {
-		$dashboard_options['kioskmode'] = true;
-	}
-
 	// Initialize dashboard grid.
 	$this->addPostJS(
 		'jQuery(".'.ZBX_STYLE_DASHBRD_GRID_CONTAINER.'")'.
