@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ $options = [
 			'show_timeline' => $data['filter']['show_timeline'],
 			'details' => $data['filter']['details'],
 			'highlight_row' => $data['filter']['highlight_row'],
-			'show_latest_values' => $data['filter']['show_latest_values']
+			'show_opdata' => $data['filter']['show_opdata']
 		]
 	]
 ];
@@ -268,7 +268,7 @@ if ($data['action'] == 'problem.view') {
 				->addValue(PROBLEMS_SHOW_TAGS_3, PROBLEMS_SHOW_TAGS_3)
 				->setModern(true)
 		)
-		->addItem((new CDiv())->setWidth(ZBX_STYLE_FORM_INPUT_MARGIN))
+		->addItem((new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN))
 		->addItem(_('Tag name'))
 		->addItem((new CRadioButtonList('filter_tag_name_format', (int) $data['filter']['tag_name_format']))
 				->addValue(_('Full'), PROBLEMS_TAG_NAME_FULL)
@@ -288,6 +288,14 @@ if ($data['action'] == 'problem.view') {
 				->setAttribute('placeholder', _('comma-separated list'))
 				->setEnabled((int) $data['filter']['show_tags'] !== PROBLEMS_SHOW_TAGS_NONE)
 		)
+		->addRow(_('Show operational data'), [
+			(new CRadioButtonList('filter_show_opdata', (int) $data['filter']['show_opdata']))
+				->addValue(_('None'), OPERATIONAL_DATA_SHOW_NONE)
+				->addValue(_('Separately'), OPERATIONAL_DATA_SHOW_SEPARATELY)
+				->addValue(_('With problem name'), OPERATIONAL_DATA_SHOW_WITH_PROBLEM)
+				->setModern(true)
+				->setEnabled($data['filter']['compact_view'] == 0)
+		])
 		->addRow(_('Show suppressed problems'), [
 			(new CCheckBox('filter_show_suppressed'))
 				->setChecked($data['filter']['show_suppressed'] == ZBX_PROBLEM_SUPPRESSED_TRUE),
@@ -319,11 +327,6 @@ if ($data['action'] == 'problem.view') {
 			]))
 				->addClass(ZBX_STYLE_FILTER_HIGHLIGHT_ROW_CB)
 				->addClass(ZBX_STYLE_TABLE_FORMS_SECOND_COLUMN)
-		])
-		->addRow(_('Show latest values'), [
-			(new CCheckBox('filter_show_latest_values'))
-				->setChecked($data['filter']['show_latest_values'] == 1)
-				->setEnabled($data['filter']['compact_view'] == 0)
 		]);
 
 	$filter = (new CFilter((new CUrl('zabbix.php'))->setArgument('action', 'problem.view')))
@@ -380,7 +383,6 @@ if ($data['action'] == 'problem.view') {
 			'mainObject' => 1
 		];
 
-		$this->addPostJS('timeControl.useTimeRefresh('.zbx_jsvalue(CWebUser::getRefresh()).');');
 		$this->addPostJS('timeControl.addObject("scroll_events_id", '.zbx_jsvalue($screen->timeline).', '.zbx_jsvalue($objData).');');
 		$this->addPostJS('timeControl.processObjects();');
 	}
