@@ -318,29 +318,6 @@ function actionConditionValueToString(array $actions, array $config) {
 	return $result;
 }
 
-/**
- * Converts numerical action operation condition values to their corresponding string values according to
- * action operation condition type. Since action list does not display operation conditions,
- * so there is only an array of operation conditions for single action which is displayed in operation details.
- *
- * @param array  $conditions					array of actions operation conditions
- * @param string $condition['conditiontype']	operation condition type
- * @param string $condition['value']			operation condition value
- *
- * @return array								returns an array of action operation condition string values
- */
-function actionOperationConditionValueToString(array $conditions) {
-	$result = [];
-
-	foreach ($conditions as $condition) {
-		if ($condition['conditiontype'] == CONDITION_TYPE_EVENT_ACKNOWLEDGED) {
-			$result[] = $condition['value'] ? _('Ack') : _('Not Ack');
-		}
-	}
-
-	return $result;
-}
-
 /**
  * Returns the HTML representation of an action condition and action operation condition.
  *
@@ -512,7 +489,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 
 	if ($media_typeids) {
 		$media_types = API::Mediatype()->get([
-			'output' => ['description'],
+			'output' => ['name'],
 			'mediatypeids' => $media_typeids,
 			'preservekeys' => true
 		]);
@@ -573,7 +550,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 						$media_typeid = $operation['opmessage']['mediatypeid'];
 
 						if ($media_typeid != 0 && isset($media_types[$media_typeid])) {
-							$media_type = $media_types[$media_typeid]['description'];
+							$media_type = $media_types[$media_typeid]['name'];
 						}
 
 						if (array_key_exists('opmessage_usr', $operation) && $operation['opmessage_usr']) {
@@ -736,7 +713,7 @@ function getActionOperationDescriptions(array $actions, $type) {
 						$media_typeid = $operation['opmessage']['mediatypeid'];
 
 						if ($media_typeid != 0 && isset($media_types[$media_typeid])) {
-							$media_type = $media_types[$media_typeid]['description'];
+							$media_type = $media_types[$media_typeid]['name'];
 						}
 
 						if (array_key_exists('opmessage_usr', $operation) && $operation['opmessage_usr']) {
@@ -1733,7 +1710,7 @@ function getEventUpdates(array $event) {
  * @param array  $actions['messages']    Messages icon data.
  * @param array  $actions['severities']  Severity change icon data.
  * @param array  $actions['actions']     Actions icon data.
- * @param array  $mediatypes             Mediatypes with maxattempts value and description.
+ * @param array  $mediatypes             Mediatypes with maxattempts value and name.
  * @param array  $users                  User name, surname and alias.
  * @param array  $config                 Zabbix config.
  *
@@ -1894,8 +1871,8 @@ function makeEventSeverityChangesIcon(array $data, array $users, array $config)
  * @param bool   $data['has_uncomplete_action']     Does the event have at least one uncompleted alert action.
  * @param bool   $data['has_failed_action']         Does the event have at least one failed alert action.
  * @param array  $users                             User name, surname and alias.
- * @param array  $mediatypes                        Mediatypes with maxattempts value and description.
- * @param string $mediatypes[]['description']       Mediatype description.
+ * @param array  $mediatypes                        Mediatypes with maxattempts value and name.
+ * @param string $mediatypes[]['name']              Mediatype name.
  * @param array  $config                            Zabbix config.
  *
  * @return CSpan|null
@@ -1924,7 +1901,7 @@ function makeEventActionsIcon(array $data, array $users, array $mediatypes, arra
 			}
 			elseif ($action['alerttype'] == ALERT_TYPE_MESSAGE) {
 				$message = array_key_exists($action['mediatypeid'], $mediatypes)
-					? $mediatypes[$action['mediatypeid']]['description']
+					? $mediatypes[$action['mediatypeid']]['name']
 					: '';
 			}
 		}
@@ -2234,8 +2211,9 @@ function makeActionTableInfo(array $action, array $mediatypes) {
 		if ($action['alerttype'] == ALERT_TYPE_MESSAGE
 				&& ($action['status'] == ALERT_STATUS_NEW || $action['status'] == ALERT_STATUS_NOT_SENT)) {
 			$info_icons[] = makeWarningIcon(array_key_exists($action['mediatypeid'], $mediatypes)
-				? _n(_('%1$s retry left'), _('%1$s retries left'),
-						$mediatypes[$action['mediatypeid']]['maxattempts'] - $action['retries'])
+				? _n('%1$s retry left', '%1$s retries left',
+						$mediatypes[$action['mediatypeid']]['maxattempts'] - $action['retries']
+				)
 				: ''
 			);
 		}
