@@ -268,7 +268,7 @@ abstract class CItemGeneral extends CApiService {
 
 				$this->checkNoParameters(
 					$item,
-					['templateid', 'state'],
+					['templateid', 'state', 'lastlogsize', 'mtime', 'error'],
 					_('Cannot update "%1$s" for item "%2$s".'),
 					$item['name']
 				);
@@ -366,22 +366,14 @@ abstract class CItemGeneral extends CApiService {
 						);
 					}
 
-					if ($fullItem['type'] == ITEM_TYPE_ZABBIX_ACTIVE) {
-						// Remove flexible and scheduling intervals and leave only the delay part.
-						$item['delay'] = $delay;
-					}
 					// If there are scheduling intervals or intervals with macros, skip the next check calculation.
-					elseif (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
+					if (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
 							&& calculateItemNextCheck(0, $delay_sec, $flexible_intervals, time()) == ZBX_JAN_2038) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
 							_('Item will not be refreshed. Please enter a correct update interval.')
 						);
 					}
 				}
-				elseif ($fullItem['type'] == ITEM_TYPE_ZABBIX_ACTIVE) {
-					// Remove flexible and scheduling intervals and leave only the delay part.
-					$item['delay'] = $delay;
-				}
 			}
 
 			// For non-numeric types, whichever value was entered in trends field, is overwritten to zero.
@@ -623,6 +615,8 @@ abstract class CItemGeneral extends CApiService {
 			}
 
 			$this->checkSpecificFields($fullItem, $update ? 'update' : 'create');
+
+			$this->validateItemPreprocessing($fullItem);
 		}
 		unset($item);
 
@@ -638,9 +632,7 @@ abstract class CItemGeneral extends CApiService {
 	 *
 	 * @return bool
 	 */
-	protected function checkSpecificFields(array $item, $method) {
-		return true;
-	}
+	abstract protected function checkSpecificFields(array $item, $method);
 
 	protected function clearValues(array $item) {
 		if (isset($item['port']) && $item['port'] != '') {
@@ -771,9 +763,7 @@ abstract class CItemGeneral extends CApiService {
 			}
 		}
 
-		if ($this instanceof CItem || $this instanceof CItemPrototype) {
-			$this->validateDependentItems($new_items);
-		}
+		$this->validateDependentItems($new_items);
 
 		// Save the new items.
 		if ($ins_items) {
@@ -829,8 +819,7 @@ abstract class CItemGeneral extends CApiService {
 	 *                                                                            prototypes.
 	 * @param string     $tpl_items[<itemid>]['applicationPrototypes'][]['name']
 	 * @param array      $tpl_items[<itemid>]['applications']                     (optional) Array of applicationids.
-	 * @param array      $tpl_items[<itemid>]['preprocessing']                    (optional) Suitable for items and item
-	 *                                                                            prototypes.
+	 * @param array      $tpl_items[<itemid>]['preprocessing']                    (optional)
 	 * @param int        $tpl_items[<itemid>]['preprocessing'][]['type']
 	 * @param string     $tpl_items[<itemid>]['preprocessing'][]['params']
 	 * @param int        $tpl_items[<itemid>]['flags']
@@ -841,8 +830,6 @@ abstract class CItemGeneral extends CApiService {
 	 * @return array an array of unsaved child items
 	 */
 	private function prepareInheritedItems(array $tpl_items, array $hostids = null) {
-		$class = get_class($this);
-
 		$itemids_by_templateid = [];
 		foreach ($tpl_items as $tpl_item) {
 			$itemids_by_templateid[$tpl_item['hostid']][] = $tpl_item['itemid'];
@@ -903,10 +890,10 @@ abstract class CItemGeneral extends CApiService {
 		}
 
 		foreach ($hostids_by_key as $key_ => $key_hostids) {
-			$sql_select = ($class === 'CItemPrototype') ? ',id.parent_itemid AS ruleid' : '';
+			$sql_select = ($this instanceof CItemPrototype) ? ',id.parent_itemid AS ruleid' : '';
 			// "LEFT JOIN" is needed to check flags on inherited and existing item, item prototype or lld rule.
 			// For example, when linking an item prototype with same key as in an item on target host or template.
-			$sql_join = ($class === 'CItemPrototype') ? ' LEFT JOIN item_discovery id ON i.itemid=id.itemid' : '';
+			$sql_join = ($this instanceof CItemPrototype) ? ' LEFT JOIN item_discovery id ON i.itemid=id.itemid' : '';
 			$db_items = DBselect(
 				'SELECT i.itemid,i.hostid,i.type,i.key_,i.flags,i.templateid'.$sql_select.
 					' FROM items i'.$sql_join.
@@ -923,7 +910,7 @@ abstract class CItemGeneral extends CApiService {
 		}
 
 		// Preparing list of application prototypes.
-		if ($class === 'CItemPrototype') {
+		if ($this instanceof CItemPrototype) {
 			$tpl_app_prototypes = [];
 			$item_prototypeids = [];
 
@@ -949,7 +936,7 @@ abstract class CItemGeneral extends CApiService {
 		}
 
 		// List of the discovery rules.
-		if ($class === 'CItemPrototype') {
+		if ($this instanceof CItemPrototype) {
 			// List of itemids without 'ruleid' property.
 			$tpl_itemids = [];
 			$tpl_ruleids = [];
@@ -1034,7 +1021,7 @@ abstract class CItemGeneral extends CApiService {
 						));
 					}
 
-					if ($class === 'CItemPrototype') {
+					if ($this instanceof CItemPrototype) {
 						$chd_ruleid = $chd_ruleids[$chd_host['hostid']][$tpl_item['ruleid']];
 						if (bccomp($chd_item['ruleid'], $chd_ruleid) != 0) {
 							self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -1053,7 +1040,7 @@ abstract class CItemGeneral extends CApiService {
 				}
 				else {
 					unset($new_item['itemid']);
-					if ($class === 'CItemPrototype') {
+					if ($this instanceof CItemPrototype) {
 						$new_item['ruleid'] = $chd_ruleids[$chd_host['hostid']][$tpl_item['ruleid']];
 					}
 				}
@@ -1073,11 +1060,15 @@ abstract class CItemGeneral extends CApiService {
 							));
 						}
 					}
+
+					if ($this instanceof CItem || $this instanceof CDiscoveryRule) {
+						if (!array_key_exists('itemid', $new_item)) {
+							$new_item['rtdata'] = true;
+						}
+					}
 				}
 
-				// For items and item prototypes.
-				if (($class === 'CItem' || $class === 'CItemPrototype')
-						&& array_key_exists('preprocessing', $new_item)) {
+				if (array_key_exists('preprocessing', $new_item)) {
 					foreach ($new_item['preprocessing'] as $preprocessing) {
 						if ($chd_item) {
 							$preprocessing['itemid'] = $chd_item['itemid'];
@@ -1088,7 +1079,7 @@ abstract class CItemGeneral extends CApiService {
 					}
 				}
 
-				if ($class === 'CItemPrototype' && array_key_exists('applicationPrototypes', $new_item)) {
+				if ($this instanceof CItemPrototype && array_key_exists('applicationPrototypes', $new_item)) {
 					foreach ($new_item['applicationPrototypes'] as &$application_prototype) {
 						$application_prototype['templateid'] =
 							$tpl_app_prototypes[$tpl_item['itemid']][$application_prototype['name']];
@@ -1120,7 +1111,7 @@ abstract class CItemGeneral extends CApiService {
 		}
 
 		// Setting item applications.
-		if ($class === 'CItem' || $class === 'CItemPrototype') {
+		if ($this instanceof CItem || $this instanceof CItemPrototype) {
 			$tpl_applicationids = [];
 			foreach ($tpl_items as $tpl_item) {
 				if (array_key_exists('applications', $tpl_item)) {
@@ -1159,11 +1150,7 @@ abstract class CItemGeneral extends CApiService {
 			}
 		}
 
-		if ($class === 'CItem' || $class === 'CItemPrototype') {
-			$new_items = $this->prepareDependentItems($tpl_items, $new_items, $hostids);
-		}
-
-		return $new_items;
+		return $this->prepareDependentItems($tpl_items, $new_items, $hostids);
 	}
 
 	/**
@@ -1222,36 +1209,69 @@ abstract class CItemGeneral extends CApiService {
 	/**
 	 * Validate item pre-processing.
 	 *
-	 * @param array  $item									An array of single item data.
-	 * @param array  $item['preprocessing']					An array of item pre-processing data.
-	 * @param string $item['preprocessing'][]['type']		The preprocessing option type. Possible values:
-	 *															1 - ZBX_PREPROC_MULTIPLIER;
-	 *															2 - ZBX_PREPROC_RTRIM;
-	 *															3 - ZBX_PREPROC_LTRIM;
-	 *															4 - ZBX_PREPROC_TRIM;
-	 *															5 - ZBX_PREPROC_REGSUB;
-	 *															6 - ZBX_PREPROC_BOOL2DEC;
-	 *															7 - ZBX_PREPROC_OCT2DEC;
-	 *															8 - ZBX_PREPROC_HEX2DEC;
-	 *															9 - ZBX_PREPROC_DELTA_VALUE;
-	 *															10 - ZBX_PREPROC_DELTA_SPEED;
-	 *															11 - ZBX_PREPROC_XPATH;
-	 *															12 - ZBX_PREPROC_JSONPATH.
-	 * @param string $item['preprocessing'][]['params']		Additional parameters used by preprocessing option. In case
-	 *														of regular expression (ZBX_PREPROC_REGSUB), multiple
-	 *														parameters are separated by LF (\n)character.
-	 * @param string $method								A string of "create" or "update" method.
+	 * @param array  $item                                             An array of single item data.
+	 * @param array  $item['preprocessing']                            An array of item pre-processing data.
+	 * @param string $item['preprocessing'][]['type']                  The preprocessing option type. Possible values:
+	 *                                                                  1 - ZBX_PREPROC_MULTIPLIER;
+	 *                                                                  2 - ZBX_PREPROC_RTRIM;
+	 *                                                                  3 - ZBX_PREPROC_LTRIM;
+	 *                                                                  4 - ZBX_PREPROC_TRIM;
+	 *                                                                  5 - ZBX_PREPROC_REGSUB;
+	 *                                                                  6 - ZBX_PREPROC_BOOL2DEC;
+	 *                                                                  7 - ZBX_PREPROC_OCT2DEC;
+	 *                                                                  8 - ZBX_PREPROC_HEX2DEC;
+	 *                                                                  9 - ZBX_PREPROC_DELTA_VALUE;
+	 *                                                                  10 - ZBX_PREPROC_DELTA_SPEED;
+	 *                                                                  11 - ZBX_PREPROC_XPATH;
+	 *                                                                  12 - ZBX_PREPROC_JSONPATH;
+	 *                                                                  13 - ZBX_PREPROC_VALIDATE_RANGE;
+	 *                                                                  14 - ZBX_PREPROC_VALIDATE_REGEX;
+	 *                                                                  15 - ZBX_PREPROC_VALIDATE_NOT_REGEX;
+	 *                                                                  16 - ZBX_PREPROC_ERROR_FIELD_JSON;
+	 *                                                                  17 - ZBX_PREPROC_ERROR_FIELD_XML;
+	 *                                                                  18 - ZBX_PREPROC_ERROR_FIELD_REGEX;
+	 *                                                                  19 - ZBX_PREPROC_THROTTLE_VALUE;
+	 *                                                                  20 - ZBX_PREPROC_THROTTLE_TIMED_VALUE;
+	 *                                                                  21 - ZBX_PREPROC_SCRIPT;
+	 *                                                                  22 - ZBX_PREPROC_PROMETHEUS_PATTERN;
+	 *                                                                  23 - ZBX_PREPROC_PROMETHEUS_TO_JSON;
+	 *                                                                  24 - ZBX_PREPROC_CSV_TO_JSON.
+	 * @param string $item['preprocessing'][]['params']                Additional parameters used by preprocessing
+	 *                                                                 option. Multiple parameters are separated by LF
+	 *                                                                 (\n) character.
+	 * @param string $item['preprocessing'][]['error_handler']         Action type used in case of preprocessing step
+	 *                                                                 failure. Possible values:
+	 *                                                                  0 - ZBX_PREPROC_FAIL_DEFAULT;
+	 *                                                                  1 - ZBX_PREPROC_FAIL_DISCARD_VALUE;
+	 *                                                                  2 - ZBX_PREPROC_FAIL_SET_VALUE;
+	 *                                                                  3 - ZBX_PREPROC_FAIL_SET_ERROR.
+	 * @param string $item['preprocessing'][]['error_handler_params']  Error handler parameters.
 	 */
-	protected function validateItemPreprocessing(array $item, $method) {
+	protected function validateItemPreprocessing(array $item) {
 		if (array_key_exists('preprocessing', $item)) {
 			if (!is_array($item['preprocessing'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 
-			$type_validator = new CLimitedSetValidator(['values' => array_keys(get_preprocessing_types(null, false))]);
+			$type_validator = new CLimitedSetValidator(['values' => $this::$supported_preprocessing_types]);
 
-			$required_fields = ['type', 'params'];
+			$error_handler_validator = new CLimitedSetValidator([
+				'values' => [ZBX_PREPROC_FAIL_DEFAULT, ZBX_PREPROC_FAIL_DISCARD_VALUE, ZBX_PREPROC_FAIL_SET_VALUE,
+					ZBX_PREPROC_FAIL_SET_ERROR
+				]
+			]);
+
+			$prometheus_pattern_parser = new CPrometheusPatternParser(['usermacros' => true,
+				'lldmacros' => ($this instanceof CItemPrototype)
+			]);
+			$prometheus_output_parser = new CPrometheusOutputParser(['usermacros' => true,
+				'lldmacros' => ($this instanceof CItemPrototype)
+			]);
+
+			$required_fields = ['type', 'params', 'error_handler', 'error_handler_params'];
 			$delta = false;
+			$throttling = false;
+			$prometheus = false;
 
 			foreach ($item['preprocessing'] as $preprocessing) {
 				$missing_keys = array_diff($required_fields, array_keys($preprocessing));
@@ -1279,6 +1299,9 @@ abstract class CItemGeneral extends CApiService {
 						)
 					);
 				}
+
+				$preprocessing['params'] = str_replace("\r\n", "\n", $preprocessing['params']);
+
 				switch ($preprocessing['type']) {
 					case ZBX_PREPROC_MULTIPLIER:
 						// Check if custom multiplier is a valid number.
@@ -1309,6 +1332,11 @@ abstract class CItemGeneral extends CApiService {
 					case ZBX_PREPROC_TRIM:
 					case ZBX_PREPROC_XPATH:
 					case ZBX_PREPROC_JSONPATH:
+					case ZBX_PREPROC_VALIDATE_REGEX:
+					case ZBX_PREPROC_VALIDATE_NOT_REGEX:
+					case ZBX_PREPROC_ERROR_FIELD_JSON:
+					case ZBX_PREPROC_ERROR_FIELD_XML:
+					case ZBX_PREPROC_SCRIPT:
 						// Check 'params' if not empty.
 						if (is_array($preprocessing['params'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
@@ -1322,6 +1350,7 @@ abstract class CItemGeneral extends CApiService {
 						break;
 
 					case ZBX_PREPROC_REGSUB:
+					case ZBX_PREPROC_ERROR_FIELD_REGEX:
 						// Check if 'params' are not empty and if second parameter contains (after \n) is not empty.
 						if (is_array($preprocessing['params'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
@@ -1348,9 +1377,52 @@ abstract class CItemGeneral extends CApiService {
 						}
 						break;
 
+					case ZBX_PREPROC_VALIDATE_RANGE:
+						if (is_array($preprocessing['params'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif (trim($preprocessing['params']) === '' || $preprocessing['params'] === null
+								|| $preprocessing['params'] === false) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'params', _('cannot be empty'))
+							);
+						}
+
+						$params = explode("\n", $preprocessing['params']);
+
+						if ($params[0] !== '' && !is_numeric($params[0])
+								&& (new CUserMacroParser())->parse($params[0]) != CParser::PARSE_SUCCESS
+								&& (!($this instanceof CItemPrototype)
+									|| ((new CLLDMacroFunctionParser())->parse($params[0]) != CParser::PARSE_SUCCESS
+										&& (new CLLDMacroParser())->parse($params[0]) != CParser::PARSE_SUCCESS))) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+								'params', _('a numeric value is expected')
+							));
+						}
+
+						if ($params[1] !== '' && !is_numeric($params[1])
+								&& (new CUserMacroParser())->parse($params[1]) != CParser::PARSE_SUCCESS
+								&& (!($this instanceof CItemPrototype)
+									|| ((new CLLDMacroFunctionParser())->parse($params[1]) != CParser::PARSE_SUCCESS
+										&& (new CLLDMacroParser())->parse($params[1]) != CParser::PARSE_SUCCESS))) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+								'params', _('a numeric value is expected')
+							));
+						}
+
+						if (is_numeric($params[0]) && is_numeric($params[1]) && $params[0] > $params[1]) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s(
+								'Incorrect value for field "%1$s": %2$s.',
+								'params',
+								_s('"%1$s" value must be less than or equal to "%2$s" value', _('min'), _('max'))
+							));
+						}
+						break;
+
 					case ZBX_PREPROC_BOOL2DEC:
 					case ZBX_PREPROC_OCT2DEC:
 					case ZBX_PREPROC_HEX2DEC:
+					case ZBX_PREPROC_THROTTLE_VALUE:
 						// Check if 'params' is empty, because it must be empty.
 						if (is_array($preprocessing['params'])) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
@@ -1361,6 +1433,15 @@ abstract class CItemGeneral extends CApiService {
 								_s('Incorrect value for field "%1$s": %2$s.', 'params', _('should be empty'))
 							);
 						}
+
+						if ($preprocessing['type'] == ZBX_PREPROC_THROTTLE_VALUE) {
+							if ($throttling) {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _('Only one throttling step is allowed.'));
+							}
+							else {
+								$throttling = true;
+							}
+						}
 						break;
 
 					case ZBX_PREPROC_DELTA_VALUE:
@@ -1384,60 +1465,285 @@ abstract class CItemGeneral extends CApiService {
 							$delta = true;
 						}
 						break;
+
+					case ZBX_PREPROC_THROTTLE_TIMED_VALUE:
+						$api_input_rules = [
+							'type' => API_TIME_UNIT,
+							'flags' => ($this instanceof CItem)
+								? API_NOT_EMPTY | API_ALLOW_USER_MACRO
+								: API_NOT_EMPTY | API_ALLOW_USER_MACRO | API_ALLOW_LLD_MACRO,
+							'in' => '1:'.ZBX_MAX_TIMESHIFT
+						];
+
+						if (!CApiInputValidator::validate($api_input_rules, $preprocessing['params'], 'params',
+								$error)) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+						}
+
+						if ($throttling) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Only one throttling step is allowed.'));
+						}
+						else {
+							$throttling = true;
+						}
+						break;
+
+					case ZBX_PREPROC_PROMETHEUS_PATTERN:
+					case ZBX_PREPROC_PROMETHEUS_TO_JSON:
+						if ($prometheus) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Only one Prometheus step is allowed.'));
+						}
+
+						$prometheus = true;
+
+						if (is_array($preprocessing['params'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+
+						if ($preprocessing['type'] == ZBX_PREPROC_PROMETHEUS_PATTERN) {
+							if ($preprocessing['params'] === '' || $preprocessing['params'] === null
+									|| $preprocessing['params'] === false) {
+								self::exception(ZBX_API_ERROR_PARAMETERS,
+									_s('Incorrect value for field "%1$s": %2$s.', 'params', _('cannot be empty'))
+								);
+							}
+
+							$params = explode("\n", $preprocessing['params']);
+
+							if ($params[0] === '') {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+									'params', _('first parameter is expected')
+								));
+							}
+
+							if ($prometheus_pattern_parser->parse($params[0]) != CParser::PARSE_SUCCESS) {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+									'params', _('invalid Prometheus pattern')
+								));
+							}
+
+							if ($params[1] !== ''
+									&& $prometheus_output_parser->parse($params[1]) != CParser::PARSE_SUCCESS) {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+									'params', _('invalid Prometheus output')
+								));
+							}
+						}
+						// Prometheus to JSON can be empty and has only one parameter.
+						elseif ($preprocessing['params'] !== '') {
+							if ($prometheus_pattern_parser->parse($preprocessing['params']) != CParser::PARSE_SUCCESS) {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+									'params', _('invalid Prometheus pattern')
+								));
+							}
+						}
+						break;
+
+					case ZBX_PREPROC_CSV_TO_JSON:
+						if (is_array($preprocessing['params'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif ($preprocessing['params'] === '' || $preprocessing['params'] === null
+								|| $preprocessing['params'] === false) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'params', _('cannot be empty'))
+							);
+						}
+
+						$params = explode("\n", $preprocessing['params']);
+
+						$params_cnt = count($params);
+						if ($params_cnt > 3) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif ($params_cnt == 1) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+								'params', _('second parameter is expected')
+							));
+						}
+						elseif ($params_cnt == 2) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+								'params', _('third parameter is expected')
+							));
+						}
+						else {
+							// Correct amount of parameters, but check if they are valid.
+
+							if (mb_strlen($params[0]) > 1) {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+									'params', _('value of first parameter is too long')
+								));
+							}
+
+							if (mb_strlen($params[1]) > 1) {
+								self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.',
+									'params', _('value of second parameter is too long')
+								));
+							}
+
+							$with_header_row_validator = new CLimitedSetValidator([
+								'values' => [ZBX_PREPROC_CSV_NO_HEADER, ZBX_PREPROC_CSV_HEADER]
+							]);
+
+							if (!$with_header_row_validator->validate($params[2])) {
+								self::exception(ZBX_API_ERROR_PARAMETERS,
+									_s('Incorrect value for field "%1$s": %2$s.', 'params',
+										_s('value of third parameter must be one of %1$s',
+											implode(', ', [ZBX_PREPROC_CSV_NO_HEADER, ZBX_PREPROC_CSV_HEADER])
+										)
+									)
+								);
+							}
+						}
+						break;
+				}
+
+				switch ($preprocessing['type']) {
+					case ZBX_PREPROC_RTRIM:
+					case ZBX_PREPROC_LTRIM:
+					case ZBX_PREPROC_TRIM:
+					case ZBX_PREPROC_THROTTLE_VALUE:
+					case ZBX_PREPROC_THROTTLE_TIMED_VALUE:
+					case ZBX_PREPROC_SCRIPT:
+						if (is_array($preprocessing['error_handler'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif ($preprocessing['error_handler'] != ZBX_PREPROC_FAIL_DEFAULT) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'error_handler',
+									_s('unexpected value "%1$s"', $preprocessing['error_handler'])
+								)
+							);
+						}
+
+						if (is_array($preprocessing['error_handler_params'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif ($preprocessing['error_handler_params'] !== ''
+								&& $preprocessing['error_handler_params'] !== null
+								&& $preprocessing['error_handler_params'] !== false) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'error_handler_params',
+									_('should be empty')
+								)
+							);
+						}
+						break;
+
+					default:
+						if (is_array($preprocessing['error_handler'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif (!$error_handler_validator->validate($preprocessing['error_handler'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'error_handler',
+									_s('unexpected value "%1$s"', $preprocessing['error_handler'])
+								)
+							);
+						}
+
+						if (is_array($preprocessing['error_handler_params'])) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+						}
+						elseif (($preprocessing['error_handler'] == ZBX_PREPROC_FAIL_DEFAULT
+									|| $preprocessing['error_handler'] == ZBX_PREPROC_FAIL_DISCARD_VALUE)
+								&& $preprocessing['error_handler_params'] !== ''
+								&& $preprocessing['error_handler_params'] !== null
+								&& $preprocessing['error_handler_params'] !== false) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'error_handler_params',
+									_('should be empty')
+								)
+							);
+						}
+						elseif ($preprocessing['error_handler'] == ZBX_PREPROC_FAIL_SET_ERROR
+								&& ($preprocessing['error_handler_params'] === ''
+									|| $preprocessing['error_handler_params'] === null
+									|| $preprocessing['error_handler_params'] === false)) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Incorrect value for field "%1$s": %2$s.', 'error_handler_params',
+									_('cannot be empty')
+								)
+							);
+						}
 				}
 			}
 		}
 	}
 
+	/**
+	 * Method validates preprocessing steps independently from other item properties.
+	 *
+	 * @param array  $preprocessing_steps    An array of item pre-processing step details.
+	 *                                       See self::validateItemPreprocessing for details.
+	 *
+	 * @return bool|string
+	 */
+	public function validateItemPreprocessingSteps(array $preprocessing_steps) {
+		try {
+			$this->validateItemPreprocessing(['preprocessing' => $preprocessing_steps]);
+
+			return true;
+		}
+		catch (APIException $error) {
+			return $error->getMessage();
+		}
+	}
+
 	/**
 	 * Insert item pre-processing data into DB.
 	 *
-	 * @param array $items							An array of items.
-	 * @param array $items[]['preprocessing']		An array of item pre-processing data.
+	 * @param array $items                     An array of items.
+	 * @param array $items[]['preprocessing']  An array of item pre-processing data.
 	 */
 	protected function createItemPreprocessing(array $items) {
 		$item_preproc = [];
-		$step = 1;
 
 		foreach ($items as $item) {
 			if (array_key_exists('preprocessing', $item)) {
+				$step = 1;
+
 				foreach ($item['preprocessing'] as $preprocessing) {
 					$item_preproc[] = [
 						'itemid' => $item['itemid'],
 						'step' => $step++,
 						'type' => $preprocessing['type'],
-						'params' => $preprocessing['params']
+						'params' => $preprocessing['params'],
+						'error_handler' => $preprocessing['error_handler'],
+						'error_handler_params' => $preprocessing['error_handler_params']
 					];
 				}
 			}
 		}
 
 		if ($item_preproc) {
-			DB::insert('item_preproc', $item_preproc);
+			DB::insertBatch('item_preproc', $item_preproc);
 		}
 	}
 
 	/**
 	 * Update item pre-processing data in DB. Delete old records and create new ones.
 	 *
-	 * @param array $items							An array of items.
-	 * @param array $items[]['preprocessing']		An array of item pre-processing data.
+	 * @param array $items                     An array of items.
+	 * @param array $items[]['preprocessing']  An array of item pre-processing data.
 	 */
 	protected function updateItemPreprocessing(array $items) {
 		$item_preproc = [];
 		$item_preprocids = [];
-		$step = 1;
 
 		foreach ($items as $item) {
 			if (array_key_exists('preprocessing', $item)) {
 				$item_preprocids[] = $item['itemid'];
+				$step = 1;
 
 				foreach ($item['preprocessing'] as $preprocessing) {
 					$item_preproc[] = [
 						'itemid' => $item['itemid'],
 						'step' => $step++,
 						'type' => $preprocessing['type'],
-						'params' => $preprocessing['params']
+						'params' => $preprocessing['params'],
+						'error_handler' => $preprocessing['error_handler'],
+						'error_handler_params' => $preprocessing['error_handler_params']
 					];
 				}
 			}
@@ -1448,7 +1754,7 @@ abstract class CItemGeneral extends CApiService {
 		}
 
 		if ($item_preproc) {
-			DB::insert('item_preproc', $item_preproc);
+			DB::insertBatch('item_preproc', $item_preproc);
 		}
 	}
 
@@ -1512,6 +1818,30 @@ abstract class CItemGeneral extends CApiService {
 			$result = $relationMap->mapMany($result, $hosts, 'hosts');
 		}
 
+		// adding preprocessing
+		if ($options['selectPreprocessing'] !== null && $options['selectPreprocessing'] != API_OUTPUT_COUNT) {
+			$db_item_preproc = API::getApiService()->select('item_preproc', [
+				'output' => $this->outputExtend($options['selectPreprocessing'], ['itemid', 'step']),
+				'filter' => ['itemid' => array_keys($result)],
+			]);
+
+			CArrayHelper::sort($db_item_preproc, ['step']);
+
+			foreach ($result as &$item) {
+				$item['preprocessing'] = [];
+			}
+			unset($item);
+
+			foreach ($db_item_preproc as $step) {
+				$itemid = $step['itemid'];
+				unset($step['item_preprocid'], $step['itemid'], $step['step']);
+
+				if (array_key_exists($itemid, $result)) {
+					$result[$itemid]['preprocessing'][] = $step;
+				}
+			}
+		}
+
 		return $result;
 	}
 
@@ -1533,7 +1863,8 @@ abstract class CItemGeneral extends CApiService {
 
 		foreach ($items as $item) {
 			if ($item['type'] == ITEM_TYPE_DEPENDENT) {
-				if ($this instanceof CItemPrototype || $item['flags'] == ZBX_FLAG_DISCOVERY_NORMAL) {
+				if ($this instanceof CDiscoveryRule || $this instanceof CItemPrototype
+						|| $item['flags'] == ZBX_FLAG_DISCOVERY_NORMAL) {
 					$dep_items[] = $item;
 				}
 
@@ -1576,17 +1907,9 @@ abstract class CItemGeneral extends CApiService {
 
 		$master_items = [];
 
-		// Fill relations array by master items (item prototypes).
+		// Fill relations array by master items (item prototypes). Discovery rule should not be master item.
 		do {
-			if ($this instanceof CItem) {
-				$db_master_items = DBselect(
-					'SELECT i.itemid,i.hostid,i.master_itemid'.
-					' FROM items i'.
-					' WHERE '.dbConditionId('i.itemid', array_keys($master_itemids)).
-						' AND '.dbConditionInt('i.flags', [ZBX_FLAG_DISCOVERY_NORMAL])
-				);
-			}
-			else {
+			if ($this instanceof CItemPrototype) {
 				$db_master_items = DBselect(
 					'SELECT i.itemid,i.hostid,i.master_itemid,i.flags,id.parent_itemid AS ruleid'.
 					' FROM items i'.
@@ -1596,6 +1919,15 @@ abstract class CItemGeneral extends CApiService {
 						' AND '.dbConditionInt('i.flags', [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_PROTOTYPE])
 				);
 			}
+			// CDiscoveryRule, CItem
+			else {
+				$db_master_items = DBselect(
+					'SELECT i.itemid,i.hostid,i.master_itemid'.
+					' FROM items i'.
+					' WHERE '.dbConditionId('i.itemid', array_keys($master_itemids)).
+						' AND '.dbConditionInt('i.flags', [ZBX_FLAG_DISCOVERY_NORMAL])
+				);
+			}
 
 			while ($db_master_item = DBfetch($db_master_items)) {
 				$master_items[$db_master_item['itemid']] = $db_master_item;
@@ -1862,23 +2194,20 @@ abstract class CItemGeneral extends CApiService {
 			],
 			'authtype' => [
 				'type' => API_INT32,
-				'in' => implode(',', [HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM])
+				'in' => implode(',', [
+					HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS
+				])
 			]
 		];
 
 		$data = $item + $db_item;
 
 		if (array_key_exists('authtype', $data)
-				&& ($data['authtype'] == HTTPTEST_AUTH_BASIC || $data['authtype'] == HTTPTEST_AUTH_NTLM)) {
+				&& ($data['authtype'] == HTTPTEST_AUTH_BASIC || $data['authtype'] == HTTPTEST_AUTH_NTLM
+					|| $data['authtype'] == HTTPTEST_AUTH_KERBEROS)) {
 			$rules += [
-				'username' => [
-					'type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY,
-					'length' => DB::getFieldLength('items', 'username')
-				],
-				'password' => [
-					'type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY,
-					'length' => DB::getFieldLength('items', 'password')
-				]
+				'username' => [ 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('items', 'username')],
+				'password' => [ 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('items', 'password')]
 			];
 		}
 
@@ -1982,11 +2311,13 @@ abstract class CItemGeneral extends CApiService {
 				libxml_clear_errors();
 
 				if (!$errors) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Cannot read XML: %1$s.', _('XML is empty')));
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Invalid parameter "%1$s": %2$s.', 'posts', _('XML is expected'))
+					);
 				}
 				else {
 					$error = reset($errors);
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Cannot read XML: %1$s.',
+					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', 'posts',
 						_s('%1$s [Line: %2$s | Column: %3$s]', '('.$error->code.') '.trim($error->message),
 						$error->line, $error->column
 					)));
@@ -1995,7 +2326,9 @@ abstract class CItemGeneral extends CApiService {
 
 			if ($data['post_type'] == ZBX_POSTTYPE_JSON) {
 				if (trim($posts, " \r\n") === '') {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot read JSON.'));
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Invalid parameter "%1$s": %2$s.', 'posts', _('JSON is expected'))
+					);
 				}
 
 				$types = [
@@ -2022,7 +2355,9 @@ abstract class CItemGeneral extends CApiService {
 				$json->decode($posts);
 
 				if ($json->hasError()) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot read JSON.'));
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Invalid parameter "%1$s": %2$s.', 'posts', _('JSON is expected'))
+					);
 				}
 			}
 		}
