@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -219,9 +219,6 @@ class CScreenBuilder {
 			case SCREEN_RESOURCE_CLOCK:
 				return new CScreenClock($options);
 
-			case SCREEN_RESOURCE_SCREEN:
-				return new CScreenScreen($options);
-
 			case SCREEN_RESOURCE_TRIGGER_OVERVIEW:
 				return new CScreenTriggersOverview($options);
 
@@ -639,15 +636,6 @@ class CScreenBuilder {
 		zbx_add_post_js('timeControl.addObject("scrollbar", '.zbx_jsvalue($timeline).', '.zbx_jsvalue($obj_data).');');
 	}
 
-	/**
-	 * Insert javascript to make time control synchronizes with NOW!
-	 *
-	 * @static
-	 */
-	public static function insertScreenRefreshTimeJs() {
-		zbx_add_post_js('timeControl.useTimeRefresh('.CWebUser::getRefresh().');');
-	}
-
 	/**
 	 * Insert javascript to init screens.
 	 *
@@ -686,7 +674,6 @@ class CScreenBuilder {
 	 */
 	public static function insertScreenStandardJs(array $timeline) {
 		CScreenBuilder::insertScreenScrollJs($timeline);
-		CScreenBuilder::insertScreenRefreshTimeJs();
 		CScreenBuilder::insertProcessObjectsJs();
 	}
 
