@@ -38,24 +38,45 @@ class CWidgetHelper {
 	 *
 	 * @param string $dialogue_name
 	 * @param string $type
+	 * @param int $view_mode (ZBX_WIDGET_VIEW_MODE_NORMAL|ZBX_WIDGET_VIEW_MODE_HIDDEN_HEADER)
 	 * @param array $known_widget_types
 	 * @param CWidgetFieldComboBox $field_rf_rate
 	 *
 	 * @return CFormList
 	 */
-	public static function createFormList($dialogue_name, $type, $known_widget_types, $field_rf_rate) {
+	public static function createFormList($dialogue_name, $type, $view_mode, $known_widget_types, $field_rf_rate) {
 		return (new CFormList())
-			->addRow((new CLabel(_('Type'), 'type')),
-				(new CComboBox('type', $type, 'updateWidgetConfigDialogue()', $known_widget_types))
+			->addRow((new CLabel(_('Type'), 'type')), [
+					(new CComboBox('type', $type, 'updateWidgetConfigDialogue()', $known_widget_types)),
+					(new CDiv((new CCheckBox('show_header'))
+						->setLabel(_('Show header'))
+						->setLabelPosition(CCheckBox::LABEL_POSITION_LEFT)
+						->setId('show_header')
+						->setChecked($view_mode == ZBX_WIDGET_VIEW_MODE_NORMAL)
+					))->addClass(ZBX_STYLE_TABLE_FORMS_SECOND_COLUMN)
+				]
 			)
 			->addRow(_('Name'),
-				(new CTextBox('name',$dialogue_name))
+				(new CTextBox('name', $dialogue_name))
 					->setAttribute('placeholder', _('default'))
 					->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			)
 			->addRow(self::getLabel($field_rf_rate), self::getComboBox($field_rf_rate));
 	}
 
+	/**
+	* Add Columns and Rows fields to the form of iterator.
+	*
+	* @param CFormList $form_list
+	* @param CWidgetFieldIntegerBox $field_columns
+	* @param CWidgetFieldIntegerBox $field_rows
+	*/
+	public static function addIteratorFields($form_list, $field_columns, $field_rows) {
+		$form_list
+			->addRow(self::getLabel($field_columns), self::getIntegerBox($field_columns))
+			->addRow(self::getLabel($field_rows), self::getIntegerBox($field_rows));
+	}
+
 	/**
 	 * Creates label linked to the field.
 	 *
@@ -68,25 +89,6 @@ class CWidgetHelper {
 			->setAsteriskMark(self::isAriaRequired($field));
 	}
 
-	/**
-	 * Creates label linked to the multiselect field.
-	 *
-	 * @param CMultiSelect $field
-	 *
-	 * @return CLabel
-	 */
-	public static function getMultiselectLabel($field) {
-		$field_name = $field->getName();
-
-		if ($field instanceof CWidgetFieldItem) {
-			$field_name .= ($field->isMultiple() ? '[]' : '');
-		} else {
-			$field_name .= '[]';
-		}
-
-		return (new CLabel($field->getLabel(), $field_name.'_ms'))->setAsteriskMark(self::isAriaRequired($field));
-	}
-
 	/**
 	 * @param CWidgetFieldComboBox $field
 	 *
@@ -139,34 +141,30 @@ class CWidgetHelper {
 	}
 
 	/**
-	 * @param CWidgetFieldTextArea $field      Widget field object.
-	 * @param string               $form_name  HTML form element name.
+	 * @param CWidgetFieldHostPatternSelect  $field      Widget field object.
+	 * @param string                         $form_name  HTML form element name.
 	 *
 	 * @return CDiv
 	 */
-	public static function getHostsPatternTextBox($field, $form_name) {
-		return (new CDiv([
-			(new CTextArea($field->getName(), self::makeStringFromChunks($field->getValue()), ['rows' => 1]))
-				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-				->setAriaRequired(self::isAriaRequired($field))
-				->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
-				->setAttribute('placeholder', $field->getPlaceholder())
-				->addClass(ZBX_STYLE_FORM_INPUT_MARGIN)
-				->addClass(ZBX_STYLE_PATTERNSELECT),
-			(new CButton($field->getName().'_select', _('Select')))
-				->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
-				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick('return PopUp("popup.generic", '.
-					CJs::encodeJson([
-						'srctbl' => 'hosts',
-						'srcfld1' => 'host',
-						'reference' => 'name',
-						'multiselect' => 1,
-						'dstfrm' => $form_name,
-						'dstfld1' => $field->getName()
-					]).', null, this);'
-				)
-		]))->addClass(CMultiSelect::ZBX_STYLE_CLASS);
+	public static function getHostPatternSelect($field, $form_name) {
+		return (new CPatternSelect([
+			'name' => $field->getName().'[]',
+			'object_name' => 'hosts',
+			'data' => $field->getValue(),
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'hosts',
+					'srcfld1' => 'hostid',
+					'dstfrm' => $form_name,
+					'dstfld1' => zbx_formatDomId($field->getName().'[]')
+				]
+			],
+			'add_post_js' => false
+		]))
+			->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
+			->setAttribute('placeholder', $field->getPlaceholder())
+			->setAriaRequired(self::isAriaRequired($field))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 	}
 
 	/**
@@ -184,99 +182,148 @@ class CWidgetHelper {
 	}
 
 	/**
-	 * @param CWidgetFieldGroup $field
+	 * Creates label linked to the multiselect field.
+	 *
+	 * @param CWidgetFieldMs $field
+	 *
+	 * @return CLabel
+	 */
+	public static function getMultiselectLabel($field) {
+		$field_name = $field->getName();
+
+		if ($field instanceof CWidgetFieldMs) {
+			$field_name .= ($field->isMultiple() ? '[]' : '');
+		}
+		else {
+			$field_name .= '[]';
+		}
+
+		return (new CLabel($field->getLabel(), $field_name.'_ms'))
+			->setAsteriskMark(self::isAriaRequired($field));
+	}
+
+	/**
+	 * @param CWidgetFieldMs $field
+	 * @param array $captions
+	 * @param string $form_name
+	 *
+	 * @return CMultiSelect
+	 */
+	private static function getMultiselectField($field, $captions, $form_name, $object_name, $popup_options) {
+		$field_name = $field->getName().($field->isMultiple() ? '[]' : '');
+
+		return (new CMultiSelect([
+			'name' => $field_name,
+			'object_name' => $object_name,
+			'multiple' => $field->isMultiple(),
+			'data' => $captions,
+			'popup' => [
+				'parameters' => [
+					'dstfrm' => $form_name,
+					'dstfld1' => zbx_formatDomId($field_name)
+				] + $popup_options
+			],
+			'add_post_js' => false
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(self::isAriaRequired($field));
+	}
+
+	/**
+	 * @param CWidgetFieldMsGroup $field
 	 * @param array $captions
 	 * @param string $form_name
 	 *
 	 * @return CMultiSelect
 	 */
 	public static function getGroup($field, $captions, $form_name) {
-		$field_name = $field->getName().'[]';
-
-		return (new CMultiSelect([
-			'name' => $field_name,
-			'object_name' => 'hostGroup',
-			'data' => $captions,
-			'popup' => [
-				'parameters' => [
-					'srctbl' => 'host_groups',
-					'srcfld1' => 'groupid',
-					'dstfrm' => $form_name,
-					'dstfld1' => zbx_formatDomId($field_name),
-					'real_hosts' => true,
-					'enrich_parent_groups' => true
-				]
-			],
-			'add_post_js' => false
-		]))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired(self::isAriaRequired($field));
+		return self::getMultiselectField($field, $captions, $form_name, 'hostGroup', [
+			'srctbl' => 'host_groups',
+			'srcfld1' => 'groupid',
+			'real_hosts' => true,
+			'enrich_parent_groups' => true
+		] + $field->getFilterParameters());
 	}
 
 	/**
-	 * @param CWidgetFieldHost $field
+	 * @param CWidgetFieldMsHost $field
 	 * @param array $captions
 	 * @param string $form_name
 	 *
 	 * @return CMultiSelect
 	 */
 	public static function getHost($field, $captions, $form_name) {
-		$field_name = $field->getName().'[]';
-
-		return (new CMultiSelect([
-			'name' => $field_name,
-			'object_name' => 'hosts',
-			'data' => $captions,
-			'popup' => [
-				'parameters' => [
-					'srctbl' => 'hosts',
-					'srcfld1' => 'hostid',
-					'dstfrm' => $form_name,
-					'dstfld1' => zbx_formatDomId($field_name)
-				]
-			],
-			'add_post_js' => false
-		]))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired(self::isAriaRequired($field));
+		return self::getMultiselectField($field, $captions, $form_name, 'hosts', [
+			'srctbl' => 'hosts',
+			'srcfld1' => 'hostid'
+		] + $field->getFilterParameters());
 	}
 
 	/**
-	 * @param CWidgetFieldItem $field
+	 * @param CWidgetFieldMsItem $field
 	 * @param array $captions
 	 * @param string $form_name
 	 *
 	 * @return CMultiSelect
 	 */
 	public static function getItem($field, $captions, $form_name) {
-		$field_name = $field->getName().($field->isMultiple() ? '[]' : '');
-
-		return (new CMultiSelect([
-			'name' => $field_name,
-			'object_name' => 'items',
-			'multiple' => $field->isMultiple(),
-			'data' => $captions,
-			'popup' => [
-				'parameters' => [
-						'srctbl' => 'items',
-						'srcfld1' => 'itemid',
-						'dstfrm' => $form_name,
-						'dstfld1' => zbx_formatDomId($field_name)
-					] + $field->getFilterParameters()
-			],
-			'add_post_js' => false
-		]))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired(self::isAriaRequired($field));
+		return self::getMultiselectField($field, $captions, $form_name, 'items', [
+			'srctbl' => 'items',
+			'srcfld1' => 'itemid',
+			'real_hosts' => true,
+			'webitems' => true
+		] + $field->getFilterParameters());
 	}
 
 	/**
-	 * @param CWidgetFieldSelectResource $field
-	 * @param array $caption
+	 * @param CWidgetFieldMsGraph $field
+	 * @param array $captions
 	 * @param string $form_name
 	 *
-	 * @return array
+	 * @return CMultiSelect
 	 */
+	public static function getGraph($field, $captions, $form_name) {
+		return self::getMultiselectField($field, $captions, $form_name, 'graphs', [
+			'srctbl' => 'graphs',
+			'srcfld1' => 'graphid',
+			'srcfld2' => 'name',
+			'real_hosts' => true,
+			'with_graphs' => true
+		] + $field->getFilterParameters());
+	}
+
+	/**
+	 * @param CWidgetFieldMsItemPrototype $field
+	 * @param array $captions
+	 * @param string $form_name
+	 *
+	 * @return CMultiSelect
+	 */
+	public static function getItemPrototype($field, $captions, $form_name) {
+		return self::getMultiselectField($field, $captions, $form_name, 'item_prototypes', [
+			'srctbl' => 'item_prototypes',
+			'srcfld1' => 'itemid',
+			'real_hosts' => true
+		] + $field->getFilterParameters());
+	}
+
+	/**
+	 * @param CWidgetFieldMsGraphPrototype $field
+	 * @param array $captions
+	 * @param string $form_name
+	 *
+	 * @return CMultiSelect
+	 */
+	public static function getGraphPrototype($field, $captions, $form_name) {
+		return self::getMultiselectField($field, $captions, $form_name, 'graph_prototypes', [
+			'srctbl' => 'graph_prototypes',
+			'srcfld1' => 'graphid',
+			'srcfld2' => 'name',
+			'real_hosts' => true,
+			'with_graph_prototypes' => true
+		] + $field->getFilterParameters());
+	}
+
 	public static function getSelectResource($field, $caption, $form_name) {
 		return [
 			(new CTextBox($field->getName().'_caption', $caption, true))
@@ -353,23 +400,39 @@ class CWidgetHelper {
 	 * @return CList
 	 */
 	public static function getSeverities($field, $config) {
-		$severities = (new CList())->addClass(ZBX_STYLE_LIST_CHECK_RADIO);
-
-		if ($field->getOrientation() == CWidgetFieldSeverities::ORIENTATION_HORIZONTAL) {
-			$severities->addClass(ZBX_STYLE_HOR_LIST);
-		}
+		$severities = [];
 
 		for ($severity = TRIGGER_SEVERITY_NOT_CLASSIFIED; $severity < TRIGGER_SEVERITY_COUNT; $severity++) {
-			$severities->addItem(
-				(new CCheckBox($field->getName().'[]', $severity))
-					->setLabel(getSeverityName($severity, $config))
-					->setId($field->getName().'_'.$severity)
-					->setChecked(in_array($severity, $field->getValue()))
+			$severities[$severity] = getSeverityName($severity, $config);
+		}
+
+		return self::getCheckBoxList($field, $severities);
+	}
+
+	/**
+	 * @param CWidgetFieldCheckBoxList $field
+	 * @param array $config
+	 *
+	 * @return CList
+	 */
+	public static function getCheckBoxList($field, array $config) {
+		$checkbox_list = (new CList())->addClass(ZBX_STYLE_LIST_CHECK_RADIO);
+
+		if ($field->getOrientation() == CWidgetFieldCheckBoxList::ORIENTATION_HORIZONTAL) {
+			$checkbox_list->addClass(ZBX_STYLE_HOR_LIST);
+		}
+
+		foreach ($config as $key => $label) {
+			$checkbox_list->addItem(
+				(new CCheckBox($field->getName().'[]', $key))
+					->setLabel($label)
+					->setId($field->getName().'_'.$key)
+					->setChecked(in_array($key, $field->getValue()))
 					->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
 			);
 		}
 
-		return $severities;
+		return $checkbox_list;
 	}
 
 	/**
@@ -494,21 +557,21 @@ class CWidgetHelper {
 	 * Function returns array containing HTML objects filled with given values. Used to generate HTML in widget
 	 * overrides field.
 	 *
-	 * @param string     $field_name
-	 * @param array      $value      Values to fill in particular data set row. See self::setValue() for detailed
-	 *                               description.
-	 * @param string     $form_name  Name of form in which data set fields resides.
-	 * @param int|string $row_num    Unique data set numeric identifier. Used to make unique field names.
+	 * @param CWidgetFieldGraphOverride  $field
+	 * @param array                      $value      Values to fill in particular data set row. See self::setValue() for
+	 *                                               detailed description.
+	 * @param string                     $form_name  Name of form in which data set fields resides.
+	 * @param int|string                 $row_num    Unique data set numeric identifier. Used to make unique field names.
 	 *
 	 * @return array
 	 */
-	public static function getGraphOverrideLayout($field_name, array $value, $form_name, $row_num) {
+	public static function getGraphOverrideLayout($field, array $value, $form_name, $row_num) {
 		$inputs = [];
 
 		// Create override optins list.
 		foreach (CWidgetFieldGraphOverride::getOverrideOptions() as $option) {
 			if (array_key_exists($option, $value)) {
-				$inputs[] = (new CVar($field_name.'['.$row_num.']['.$option.']', $value[$option]));
+				$inputs[] = (new CVar($field->getName().'['.$row_num.']['.$option.']', $value[$option]));
 			}
 		}
 
@@ -522,54 +585,51 @@ class CWidgetHelper {
 					->addClass(ZBX_STYLE_DRAG_ICON)
 					->addStyle('position: absolute; margin-left: -25px;'),
 				(new CDiv([
-					(new CDiv([
-						(new CTextArea($field_name.'['.$row_num.'][hosts]', self::makeStringFromChunks($value['hosts']),
-								['rows' => 1]
-							))
-							->setAttribute('placeholder', _('host pattern'))
-							->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-							->addClass(ZBX_STYLE_FORM_INPUT_MARGIN)
-							->addClass(ZBX_STYLE_PATTERNSELECT),
-						(new CButton(null, _('Select')))
-							->addClass(ZBX_STYLE_BTN_GREY)
-							->onClick('return PopUp("popup.generic", jQuery.extend('.
-								CJs::encodeJson([
+					(new CDiv(
+						(new CPatternSelect([
+							'name' => $field->getName().'['.$row_num.'][hosts][]',
+							'object_name' => 'hosts',
+							'data' => $value['hosts'],
+							'placeholder' => _('host pattern'),
+							'popup' => [
+								'parameters' => [
 									'srctbl' => 'hosts',
-									'srcfld1' => 'host',
-									'reference' => 'name',
-									'multiselect' => 1,
-									'dstfrm' => $form_name
-								]).', {dstfld1: jQuery(this).siblings("textarea").attr("name")}), null, this);'
-							)
-					]))
-						->addClass(CMultiSelect::ZBX_STYLE_CLASS)
-						->addClass(ZBX_STYLE_COLUMN_50),
-					(new CDiv([
-						(new CTextArea($field_name.'['.$row_num.'][items]', self::makeStringFromChunks($value['items']),
-								['rows' => 1]
-							))
-							->setAttribute('placeholder', _('item pattern'))
+									'srcfld1' => 'hostid',
+									'dstfrm' => $form_name,
+									'dstfld1' => zbx_formatDomId($field->getName().'['.$row_num.'][hosts][]')
+								]
+							],
+							'add_post_js' => false
+						]))
+							->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
+							->setAriaRequired(self::isAriaRequired($field))
 							->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-							->addClass(ZBX_STYLE_FORM_INPUT_MARGIN)
-							->addClass(ZBX_STYLE_PATTERNSELECT),
-						(new CButton(null, _('Select')))
-							->addClass(ZBX_STYLE_BTN_GREY)
-							->onClick('return PopUp("popup.generic", jQuery.extend('.
-								CJs::encodeJson([
+					))->addClass(ZBX_STYLE_COLUMN_50),
+					(new CDiv(
+						(new CPatternSelect([
+							'name' => $field->getName().'['.$row_num.'][items][]',
+							'object_name' => 'items',
+							'data' => $value['items'],
+							'placeholder' => _('item pattern'),
+							'multiple' => true,
+							'popup' => [
+								'parameters' => [
 									'srctbl' => 'items',
 									'srcfld1' => 'itemid',
-									'reference' => 'name_expanded',
-									'multiselect' => 1,
 									'real_hosts' => 1,
 									'numeric' => 1,
-									'with_webitems' => 1,
+									'webitems' => 1,
 									'orig_names' => 1,
-									'dstfrm' => $form_name
-								]).', {dstfld1: jQuery(this).siblings("textarea").attr("name")}), null, this);'
-							)
-					]))
-						->addClass(CMultiSelect::ZBX_STYLE_CLASS)
-						->addClass(ZBX_STYLE_COLUMN_50)
+									'dstfrm' => $form_name,
+									'dstfld1' => zbx_formatDomId($field->getName().'['.$row_num.'][items][]')
+								]
+							],
+							'add_post_js' => false
+						]))
+							->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
+							->setAriaRequired(self::isAriaRequired($field))
+							->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+					))->addClass(ZBX_STYLE_COLUMN_50)
 				]))
 					->addClass(ZBX_STYLE_COLUMN_95)
 					->addClass(ZBX_STYLE_COLUMNS),
@@ -608,7 +668,7 @@ class CWidgetHelper {
 	public static function getGraphOverrideTemplate($field, $form_name) {
 		$value = CWidgetFieldGraphOverride::getDefaults();
 
-		return self::getGraphOverrideLayout($field->getName(), $value, $form_name, '#{rowNum}')->toString();
+		return self::getGraphOverrideLayout($field, $value, $form_name, '#{rowNum}')->toString();
 	}
 
 	/**
@@ -630,7 +690,7 @@ class CWidgetHelper {
 		$i = 0;
 
 		foreach ($values as $override) {
-			$list->addItem(self::getGraphOverrideLayout($field->getName(), $override, $form_name, $i));
+			$list->addItem(self::getGraphOverrideLayout($field, $override, $form_name, $i));
 
 			$i++;
 		}
@@ -660,6 +720,7 @@ class CWidgetHelper {
 			'type'.SVG_GRAPH_TYPE_LINE => _('Line'),
 			'type'.SVG_GRAPH_TYPE_POINTS => _('Points'),
 			'type'.SVG_GRAPH_TYPE_STAIRCASE => _('Staircase'),
+			'type'.SVG_GRAPH_TYPE_BAR => _('Bar'),
 			'transparency' => _('Transparency'),
 			'fill' => _('Fill'),
 			'pointsize' => _('Point size'),
@@ -702,6 +763,7 @@ class CWidgetHelper {
 						['name' => _('Draw').'/'._('Line'), 'callback' => 'addOverride', 'args' => ['type', SVG_GRAPH_TYPE_LINE]],
 						['name' => _('Draw').'/'._('Points'), 'callback' => 'addOverride', 'args' => ['type', SVG_GRAPH_TYPE_POINTS]],
 						['name' => _('Draw').'/'._('Staircase'), 'callback' => 'addOverride', 'args' => ['type', SVG_GRAPH_TYPE_STAIRCASE]],
+						['name' => _('Draw').'/'._('Bar'), 'callback' => 'addOverride', 'args' => ['type', SVG_GRAPH_TYPE_BAR]],
 
 						['name' => _('Transparency').'/0', 'callback' => 'addOverride', 'args' => ['transparency', 0]],
 						['name' => _('Transparency').'/1', 'callback' => 'addOverride', 'args' => ['transparency', 1]],
@@ -796,15 +858,11 @@ class CWidgetHelper {
 					'var container = jQuery(".overlay-dialogue-body");'.
 					'container.scrollTop(container[0].scrollHeight);'.
 
-					// Initialize textarea autogrow.
-					'jQuery("textarea", jQuery("#overrides"))'.
-						'.filter(function() {return this.id.match(/or_\d+_hosts/);})'.
-						'.each(function() {'.
-							'var itemsId = jQuery(this).attr("id").replace("_hosts", "_items"),'.
-								'hostsId = jQuery(this).attr("id");'.
-							'jQuery(this).autoGrowTextarea({pair: "#"+itemsId, maxHeight: 100});'.
-							'jQuery("#"+itemsId).autoGrowTextarea({pair: "#"+hostsId, maxHeight: 100});'.
-						'});'.
+					'jQuery(".multiselect", jQuery("#overrides")).each(function() {'.
+						'jQuery(this).multiSelect(jQuery(this).data("params"));'.
+					'});'.
+					'updateVariableOrder(jQuery("#overrides"), ".'.ZBX_STYLE_OVERRIDES_LIST_ITEM.'", "or");'.
+					'onGraphConfigChange();'.
 				'})'.
 				'.bind("afterremove.dynamicRows", function(event, options) {'.
 					'updateVariableOrder(jQuery("#overrides"), ".'.ZBX_STYLE_OVERRIDES_LIST_ITEM.'", "or");'.
@@ -826,15 +884,10 @@ class CWidgetHelper {
 			// Initialize overrides UI control.
 			'initializeOverrides();',
 
-			// Initialize textarea autogrow.
-			'jQuery("textarea", jQuery("#overrides"))'.
-				'.filter(function() {return this.id.match(/or_\d+_hosts/);})'.
-				'.each(function() {'.
-					'var itemsId = jQuery(this).attr("id").replace("_hosts", "_items"),'.
-						'hostsId = jQuery(this).attr("id");'.
-					'jQuery(this).autoGrowTextarea({pair: "#"+itemsId, maxHeight: 100});'.
-					'jQuery("#"+itemsId).autoGrowTextarea({pair: "#"+hostsId, maxHeight: 100});'.
-				'});',
+			// Initialize override pattern-selectors.
+			'jQuery(".multiselect", jQuery("#overrides")).each(function() {'.
+				'jQuery(this).multiSelect(jQuery(this).data("params"));'.
+			'});',
 
 			// Make overrides sortable.
 			'if (jQuery("#overrides .'.ZBX_STYLE_OVERRIDES_LIST_ITEM.'").length < 2) {'.
@@ -891,53 +944,43 @@ class CWidgetHelper {
 							->addClass(ZBX_STYLE_COLOR_PREVIEW_BOX)
 							->addStyle('background-color: #'.$value['color'].';')
 							->setAttribute('title', $is_opened ? _('Collapse') : _('Expand')),
-						(new CDiv([
-							(new CTextArea($field_name.'['.$row_num.'][hosts]',
-								self::makeStringFromChunks($value['hosts']), ['rows' => 1])
-							)
-								->setAttribute('placeholder', _('host pattern'))
-								->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-								->addClass(ZBX_STYLE_FORM_INPUT_MARGIN)
-								->addClass(ZBX_STYLE_PATTERNSELECT),
-							(new CButton(null, _('Select')))
-								->addClass(ZBX_STYLE_BTN_GREY)
-								->onClick('return PopUp("popup.generic", jQuery.extend('.
-									CJs::encodeJson([
-										'srctbl' => 'hosts',
-										'srcfld1' => 'host',
-										'reference' => 'name',
-										'multiselect' => 1,
-										'dstfrm' => $form_name
-									]).', {dstfld1: jQuery(this).siblings("textarea").attr("name")}), null, this);'
-								)
-						]))->addClass(CMultiSelect::ZBX_STYLE_CLASS)
+						(new CPatternSelect([
+							'name' => $field_name.'['.$row_num.'][hosts][]',
+							'object_name' => 'hosts',
+							'data' => $value['hosts'],
+							'placeholder' => _('host pattern'),
+							'popup' => [
+								'parameters' => [
+									'srctbl' => 'hosts',
+									'srcfld1' => 'host',
+									'dstfrm' => $form_name,
+									'dstfld1' => zbx_formatDomId($field_name.'['.$row_num.'][hosts][]')
+								]
+							],
+							'add_post_js' => false
+						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 					]))->addClass(ZBX_STYLE_COLUMN_50),
-					(new CDiv([
-						(new CTextArea($field_name.'['.$row_num.'][items]',
-							self::makeStringFromChunks($value['items']), ['rows' => 1])
-						)
-							->setAttribute('placeholder', _('item pattern'))
-							->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-							->addClass(ZBX_STYLE_FORM_INPUT_MARGIN)
-							->addClass(ZBX_STYLE_PATTERNSELECT),
-						(new CButton(null, _('Select')))
-							->addClass(ZBX_STYLE_BTN_GREY)
-							->onClick('return PopUp("popup.generic", jQuery.extend('.
-								CJs::encodeJson([
+					(new CDiv(
+						(new CPatternSelect([
+							'name' => $field_name.'['.$row_num.'][items][]',
+							'object_name' => 'items',
+							'data' => $value['items'],
+							'placeholder' => _('item pattern'),
+							'popup' => [
+								'parameters' => [
 									'srctbl' => 'items',
-									'srcfld1' => 'itemid',
-									'reference' => 'name_expanded',
-									'multiselect' => 1,
+									'srcfld1' => 'name',
 									'real_hosts' => 1,
 									'numeric' => 1,
-									'with_webitems' => 1,
+									'webitems' => 1,
 									'orig_names' => 1,
-									'dstfrm' => $form_name
-								]).', {dstfld1: jQuery(this).siblings("textarea").attr("name")}), null, this);'
-							)
-					]))
-						->addClass(CMultiSelect::ZBX_STYLE_CLASS)
-						->addClass(ZBX_STYLE_COLUMN_50),
+									'dstfrm' => $form_name,
+									'dstfld1' => zbx_formatDomId($field_name.'['.$row_num.'][items][]')
+								]
+							],
+							'add_post_js' => false
+						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+					))->addClass(ZBX_STYLE_COLUMN_50),
 				]))
 					->addClass(ZBX_STYLE_COLUMN_95)
 					->addClass(ZBX_STYLE_COLUMNS),
@@ -965,12 +1008,13 @@ class CWidgetHelper {
 									->addValue(_('Line'), SVG_GRAPH_TYPE_LINE)
 									->addValue(_('Points'), SVG_GRAPH_TYPE_POINTS)
 									->addValue(_('Staircase'), SVG_GRAPH_TYPE_STAIRCASE)
+									->addValue(_('Bar'), SVG_GRAPH_TYPE_BAR)
 									->onChange('changeDataSetDrawType(this)')
 									->setModern(true)
 							)
 							->addRow(_('Width'),
 								(new CRangeControl($field_name.'['.$row_num.'][width]', (int) $value['width']))
-									->setEnabled($value['type'] != SVG_GRAPH_TYPE_POINTS)
+									->setEnabled(!in_array($value['type'], [SVG_GRAPH_TYPE_POINTS, SVG_GRAPH_TYPE_BAR]))
 									->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 									->setStep(1)
 									->setMin(0)
@@ -995,7 +1039,7 @@ class CWidgetHelper {
 							)
 							->addRow(_('Fill'),
 								(new CRangeControl($field_name.'['.$row_num.'][fill]', (int) $value['fill']))
-									->setEnabled($value['type'] != SVG_GRAPH_TYPE_POINTS)
+									->setEnabled(!in_array($value['type'], [SVG_GRAPH_TYPE_POINTS, SVG_GRAPH_TYPE_BAR]))
 									->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 									->setStep(1)
 									->setMin(0)
@@ -1019,7 +1063,7 @@ class CWidgetHelper {
 									->addValue(_x('Treat as 0', 'missing data function'),
 										SVG_GRAPH_MISSING_DATA_TREAT_AS_ZERO
 									)
-									->setEnabled($value['type'] != SVG_GRAPH_TYPE_POINTS)
+									->setEnabled(!in_array($value['type'], [SVG_GRAPH_TYPE_POINTS, SVG_GRAPH_TYPE_BAR]))
 									->setModern(true)
 							)
 							->addRow(_('Y-axis'),
@@ -1033,6 +1077,43 @@ class CWidgetHelper {
 									->setAttribute('placeholder', _('none'))
 									->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 							)
+							->addRow(_('Aggregation function'),
+								(new CComboBox(
+									$field_name.'['.$row_num.'][aggregate_function]',
+									(int) $value['aggregate_function'], null,
+									[
+										GRAPH_AGGREGATE_NONE => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_NONE),
+										GRAPH_AGGREGATE_MIN => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_MIN),
+										GRAPH_AGGREGATE_MAX => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_MAX),
+										GRAPH_AGGREGATE_AVG => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_AVG),
+										GRAPH_AGGREGATE_COUNT => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_COUNT),
+										GRAPH_AGGREGATE_SUM => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_SUM),
+										GRAPH_AGGREGATE_FIRST => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_FIRST),
+										GRAPH_AGGREGATE_LAST => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_LAST)
+									]
+								))
+									->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
+									->onChange('changeDataSetAggregateFunction(this);')
+							)
+							->addRow(_('Aggregation interval'),
+								(new CTextBox(
+									$field_name.'['.$row_num.'][aggregate_interval]',
+									$value['aggregate_interval']
+								))
+									->setEnabled($value['aggregate_function'] != GRAPH_AGGREGATE_NONE)
+									->setAttribute('placeholder', GRAPH_AGGREGATE_DEFAULT_INTERVAL)
+									->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
+							)
+							->addRow(_('Aggregate'),
+								(new CRadioButtonList(
+									$field_name.'['.$row_num.'][aggregate_grouping]',
+									(int) $value['aggregate_grouping'])
+								)
+									->addValue(_('Each item'), GRAPH_AGGREGATE_BY_ITEM)
+									->addValue(_('Data set'), GRAPH_AGGREGATE_BY_DATASET)
+									->setEnabled($value['aggregate_function'] != GRAPH_AGGREGATE_NONE)
+									->setModern(true)
+							)
 					))
 						->addClass(ZBX_STYLE_COLUMN_50),
 				]))
@@ -1073,18 +1154,11 @@ class CWidgetHelper {
 		$values = $field->getValue();
 
 		if (!$values) {
-			$values[] = [];
+			$values[] = CWidgetFieldGraphDataSet::getDefaults();
 		}
 
-		$i = 0;
-
-		foreach ($values as $value) {
-			// Take default values for missing fields. This can happen if particular field is disabled.
-			$value += CWidgetFieldGraphDataSet::getDefaults();
-
+		foreach ($values as $i => $value) {
 			$list->addItem(self::getGraphDataSetLayout($field->getName(), $value, $form_name, $i, $i == 0));
-
-			$i++;
 		}
 
 		// Add 'Add' button under accordion.
@@ -1109,24 +1183,45 @@ class CWidgetHelper {
 		$scripts = [
 			'function changeDataSetDrawType(obj) {'.
 				'var row_num = obj.id.replace("ds_", "").replace("_type", "");'.
-				'if (jQuery(":checked", jQuery(obj)).val() == '.SVG_GRAPH_TYPE_POINTS.') {'.
-					'jQuery("#ds_" + row_num + "_width").rangeControl("disable");'.
-					'jQuery("#ds_" + row_num + "_fill").rangeControl("disable");'.
-					'jQuery("#ds_" + row_num + "_pointsize").rangeControl("enable");'.
-					'jQuery("#ds_" + row_num + "_missingdatafunc_0").prop("disabled", true);'.
-					'jQuery("#ds_" + row_num + "_missingdatafunc_1").prop("disabled", true);'.
-					'jQuery("#ds_" + row_num + "_missingdatafunc_2").prop("disabled", true);'.
-				'}'.
-				'else {'.
-					'jQuery("#ds_" + row_num + "_width").rangeControl("enable");'.
-					'jQuery("#ds_" + row_num + "_fill").rangeControl("enable");'.
-					'jQuery("#ds_" + row_num + "_pointsize").rangeControl("disable");'.
-					'jQuery("#ds_" + row_num + "_missingdatafunc_0").prop("disabled", false);'.
-					'jQuery("#ds_" + row_num + "_missingdatafunc_1").prop("disabled", false);'.
-					'jQuery("#ds_" + row_num + "_missingdatafunc_2").prop("disabled", false);'.
+				'switch (jQuery(":checked", jQuery(obj)).val()) {'.
+					'case "'.SVG_GRAPH_TYPE_POINTS.'":'.
+						'jQuery("#ds_" + row_num + "_width").rangeControl("disable");'.
+						'jQuery("#ds_" + row_num + "_pointsize").rangeControl("enable");'.
+						'jQuery("#ds_" + row_num + "_transparency").rangeControl("enable");'.
+						'jQuery("#ds_" + row_num + "_fill").rangeControl("disable");'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_0").prop("disabled", true);'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_1").prop("disabled", true);'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_2").prop("disabled", true);'.
+						'break;'.
+					'case "'.SVG_GRAPH_TYPE_BAR.'":'.
+						'jQuery("#ds_" + row_num + "_width").rangeControl("disable");'.
+						'jQuery("#ds_" + row_num + "_pointsize").rangeControl("disable");'.
+						'jQuery("#ds_" + row_num + "_transparency").rangeControl("enable");'.
+						'jQuery("#ds_" + row_num + "_fill").rangeControl("disable");'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_0").prop("disabled", true);'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_1").prop("disabled", true);'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_2").prop("disabled", true);'.
+						'break;'.
+					'default:'.
+						'jQuery("#ds_" + row_num + "_width").rangeControl("enable");'.
+						'jQuery("#ds_" + row_num + "_pointsize").rangeControl("disable");'.
+						'jQuery("#ds_" + row_num + "_transparency").rangeControl("enable");'.
+						'jQuery("#ds_" + row_num + "_fill").rangeControl("enable");'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_0").prop("disabled", false);'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_1").prop("disabled", false);'.
+						'jQuery("#ds_" + row_num + "_missingdatafunc_2").prop("disabled", false);'.
+						'break;'.
 				'}'.
 			'};',
 
+			'function changeDataSetAggregateFunction(obj) {'.
+				'var row_num = obj.id.replace("ds_", "").replace("_aggregate_function", "");'.
+				'var no_aggregation = (jQuery(obj).val() == '.GRAPH_AGGREGATE_NONE.');'.
+				'jQuery("#ds_" + row_num + "_aggregate_interval").prop("disabled", no_aggregation);'.
+				'jQuery("#ds_" + row_num + "_aggregate_grouping_0").prop("disabled", no_aggregation);'.
+				'jQuery("#ds_" + row_num + "_aggregate_grouping_1").prop("disabled", no_aggregation);'.
+			'};',
+
 			// Initialize dynamic rows.
 			'jQuery("#data_sets")'.
 				'.dynamicRows({'.
@@ -1155,14 +1250,11 @@ class CWidgetHelper {
 						'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
 					'}, appendTo: "#overlay_dialogue"});'.
 
-					'jQuery("textarea", jQuery("#data_sets"))'.
-						'.filter(function() {return this.id.match(/ds_\d+_hosts/);})'.
-						'.each(function() {'.
-							'var itemsId = jQuery(this).attr("id").replace("_hosts", "_items"),'.
-								'hostsId = jQuery(this).attr("id");'.
-							'jQuery(this).autoGrowTextarea({pair: "#"+itemsId, maxHeight: 100});'.
-							'jQuery("#"+itemsId).autoGrowTextarea({pair: "#"+hostsId, maxHeight: 100});'.
-						'});'.
+					'jQuery(".multiselect", jQuery("#data_sets")).each(function() {'.
+						'jQuery(this).multiSelect(jQuery(this).data("params"));'.
+					'});'.
+					'updateVariableOrder(jQuery("#data_sets"), ".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'", "ds");'.
+					'onGraphConfigChange();'.
 				'})'.
 				'.bind("afterremove.dynamicRows", function(event, options) {'.
 					'updateVariableOrder(jQuery("#data_sets"), ".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'", "ds");'.
@@ -1191,22 +1283,17 @@ class CWidgetHelper {
 
 			// Expand dataset when click in pattern fields.
 			'jQuery("#data_sets").on("click", "'.implode(', ', [
-				'.'.ZBX_STYLE_LIST_ACCORDION_ITEM_CLOSED.' .'.ZBX_STYLE_PATTERNSELECT,
+				'.'.ZBX_STYLE_LIST_ACCORDION_ITEM_CLOSED.' .'.CPatternSelect::ZBX_STYLE_CLASS,
 				'.'.ZBX_STYLE_LIST_ACCORDION_ITEM_CLOSED.' .'.ZBX_STYLE_BTN_GREY
 			]).'", function() {'.
 				'var index = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'").index();'.
 				'jQuery("#data_sets").zbx_vertical_accordion("expandNth", index);'.
 			'});',
 
-			// Initialize textarea autogrow.
-			'jQuery("textarea", jQuery("#data_sets"))'.
-				'.filter(function() {return this.id.match(/ds_\d+_hosts/);})'.
-				'.each(function() {'.
-					'var itemsId = jQuery(this).attr("id").replace("_hosts", "_items"),'.
-						'hostsId = jQuery(this).attr("id");'.
-					'jQuery(this).autoGrowTextarea({pair: "#"+itemsId, maxHeight: 100});'.
-					'jQuery("#"+itemsId).autoGrowTextarea({pair: "#"+hostsId, maxHeight: 100});'.
-				'});',
+			// Initialize pattern fields.
+			'jQuery(".multiselect", jQuery("#data_sets")).each(function() {'.
+				'jQuery(this).multiSelect(jQuery(this).data("params"));'.
+			'});',
 
 			// Initialize color-picker UI elements.
 			'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color){'.
@@ -1251,28 +1338,4 @@ class CWidgetHelper {
 	public static function isAriaRequired($field) {
 		return ($field->getFlags() & CWidgetField::FLAG_LABEL_ASTERISK);
 	}
-
-	/**
-	 * Make string from chunks.
-	 *
-	 * @param array  $chunks          Array of strings.
-	 *
-	 * @return string
-	 */
-	public static function makeStringFromChunks(array $chunks) {
-		ksort($chunks);
-		return implode('', $chunks);
-	}
-
-	/**
-	 * Make array of patterns.
-	 *
-	 * @param array|string $patterns  Comma separated string of patterns or array containing multiple patterns.
-	 *
-	 * @return array  Returns array of unique patterns.
-	 */
-	public static function splitPatternIntoParts($patterns) {
-		$patterns = is_array($patterns) ? $patterns : str_split(str_replace("\r", '', trim($patterns)), 255);
-		return array_filter($patterns, 'strlen');
-	}
 }
