@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@ if ($data['triggerid'] !== null) {
 
 if ($data['limited']) {
 	$triggersForm
+		->addVar('opdata', $data['opdata'])
 		->addVar('recovery_mode', $data['recovery_mode'])
 		->addVar('type', $data['type'])
 		->addVar('correlation_mode', $data['correlation_mode'])
@@ -55,14 +56,22 @@ $triggersFormList = new CFormList('triggersFormList');
 if (!empty($data['templates'])) {
 	$triggersFormList->addRow(_('Parent triggers'), $data['templates']);
 }
-$triggersFormList->addRow(
-	(new CLabel(_('Name'), 'description'))->setAsteriskMark(),
-	(new CTextBox('description', $data['description'], $data['limited']))
-		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		->setAriaRequired()
-		->setAttribute('autofocus', 'autofocus')
+$triggersFormList
+	->addRow(
+		(new CLabel(_('Name'), 'description'))->setAsteriskMark(),
+		(new CTextBox('description', $data['description'], $data['limited']))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
+			->setAttribute('autofocus', 'autofocus')
 	)
-	->addRow(_('Severity'), new CSeverity(['name' => 'priority', 'value' => (int) $data['priority']]));
+	->addRow(
+		new CLabel(_('Operational data'), 'opdata'),
+		(new CTextBox('opdata', $data['opdata'], $data['limited']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	)
+	->addRow(
+		_('Severity'),
+		new CSeverity(['name' => 'priority', 'value' => (int) $data['priority']])
+	);
 
 // append expression to form list
 if ($data['expression_field_readonly']) {
@@ -100,6 +109,7 @@ $expression_row = [
 		$data['expression_field_value'],
 		['readonly' => $data['expression_field_readonly']]
 	))
+		->addClass(ZBX_STYLE_MONOSPACE_FONT)
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -301,6 +311,7 @@ $recovery_expression_row = [
 		$data['recovery_expression_field_value'],
 		['readonly' => $data['recovery_expression_field_readonly']]
 	))
+		->addClass(ZBX_STYLE_MONOSPACE_FONT)
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -485,44 +496,13 @@ $triggersFormList
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			->setAriaRequired(),
 		'correlation_tag_row'
+	)
+	->addRow(_('Allow manual close'),
+		(new CCheckBox('manual_close'))
+			->setChecked($data['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED)
+			->setEnabled(!$data['limited'])
 	);
 
-// tags
-$tags_table = (new CTable())->setId('tbl_tags');
-
-foreach ($data['tags'] as $tag_key => $tag) {
-	$tags_table->addRow([
-		(new CTextBox('tags['.$tag_key.'][tag]', $tag['tag'], false, 255))
-			->setWidth(ZBX_TEXTAREA_TAG_WIDTH)
-			->setAttribute('placeholder', _('tag')),
-		(new CTextBox('tags['.$tag_key.'][value]', $tag['value'], false, 255))
-			->setWidth(ZBX_TEXTAREA_TAG_WIDTH)
-			->setAttribute('placeholder', _('value')),
-		(new CCol(
-			(new CButton('tags['.$tag_key.'][remove]', _('Remove')))
-				->addClass(ZBX_STYLE_BTN_LINK)
-				->addClass('element-table-remove')
-		))->addClass(ZBX_STYLE_NOWRAP)
-	], 'form_row');
-}
-
-$tags_table->setFooter(new CCol(
-	(new CButton('tag_add', _('Add')))
-		->addClass(ZBX_STYLE_BTN_LINK)
-		->addClass('element-table-add')
-));
-$triggersFormList->addRow(_('Tags'),
-	(new CDiv($tags_table))
-		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
-);
-
-$triggersFormList->addRow(_('Allow manual close'),
-	(new CCheckBox('manual_close'))
-		->setChecked($data['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED)
-		->setEnabled(!$data['limited'])
-);
-
 // append status to form list
 if (empty($data['triggerid']) && empty($data['form_refresh'])) {
 	$status = true;
@@ -534,7 +514,9 @@ else {
 $triggersFormList
 	->addRow(_('URL'), (new CTextBox('url', $data['url']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH))
 	->addRow(_('Description'),
-		(new CTextArea('comments', $data['comments']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		(new CTextArea('comments', $data['comments']))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setMaxlength(DB::getFieldLength('triggers', 'comments'))
 	)
 	->addRow(_('Create enabled'), (new CCheckBox('status'))->setChecked($status));
 
@@ -545,6 +527,17 @@ if (!$data['form_refresh']) {
 }
 $triggersTab->addTab('triggersTab',	_('Trigger prototype'), $triggersFormList);
 
+/*
+ * Tags tab
+ */
+$tags_view = new CView('configuration.tags.tab', [
+	'source' => 'trigger_prototype',
+	'tags' => $data['tags'],
+	'show_inherited_tags' => $data['show_inherited_tags'],
+	'readonly' => false
+]);
+$triggersTab->addTab('tags-tab', _('Tags'), $tags_view->render());
+
 /*
  * Dependencies tab
  */
