@@ -54,7 +54,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapElementFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Map element'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('formDragHandler')
 			->toString()
 	?>
@@ -154,7 +153,6 @@ function getFontComboBox($name) {
 					], 'triggerListRow')
 					->addRow((new CLabel(_('New triggers'), 'elementNameTriggers_ms')),
 						(new CDiv([
-							new CVar('elementExpressionTrigger', ''),
 							(new CMultiSelect([
 								'name' => 'elementNameTriggers',
 								'object_name' => 'triggers',
@@ -287,7 +285,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapShapeFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Map shape'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('shapeDragHandler')
 			->toString().
 		(new CForm())
@@ -436,7 +433,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapMassShapeFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Mass update shapes'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('massShapeDragHandler')
 			->toString().
 		(new CForm())
@@ -557,7 +553,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapMassFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Mass update elements'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('massDragHandler')
 			->toString()
 	?>
@@ -833,7 +828,6 @@ function getFontComboBox($name) {
 				(new CVar('element_id[#{triggerid}]', '#{triggerid}')),
 				(new CVar('element_name[#{triggerid}]', '#{name}')),
 				(new CVar('element_priority[#{triggerid}]', '#{priority}')),
-				(new CVar('element_expression[#{triggerid}]', '#{expression}')),
 				(new CButton(null, _('Remove')))
 					->addClass(ZBX_STYLE_BTN_LINK)
 					->addStyle('margin: 0 5px;')
